@@ -79,6 +79,7 @@ typedef struct
   ply_throbber_t *throbber;
   ply_progress_bar_t *progress_bar;
   ply_label_t *label;
+  ply_label_t *message_label;
   ply_rectangle_t box_area, lock_area, logo_area, bar_area;
 } view_t;
 
@@ -94,6 +95,9 @@ struct _ply_boot_splash_plugin
   char *image_dir;
   ply_boot_splash_display_type_t state;
 
+  uint32_t background_start_color;
+  uint32_t background_end_color;
+
   ply_trigger_t *idle_trigger;
 
   uint32_t root_is_mounted : 1;
@@ -101,6 +105,7 @@ struct _ply_boot_splash_plugin
   uint32_t is_animating : 1;
 };
 
+ply_boot_splash_plugin_interface_t * ply_boot_splash_plugin_get_interface (void);
 static void detach_from_event_loop (ply_boot_splash_plugin_t *plugin);
 
 static view_t *
@@ -118,6 +123,9 @@ view_new (ply_boot_splash_plugin_t *plugin,
                                      "throbber-");
   view->progress_bar = ply_progress_bar_new ();
   view->label = ply_label_new ();
+  view->message_label = ply_label_new ();
+  ply_label_set_text (view->message_label, "");
+  ply_label_show (view->message_label, view->display, 10, 10);
 
   return view;
 }
@@ -130,6 +138,7 @@ view_free (view_t *view)
   ply_throbber_free (view->throbber);
   ply_progress_bar_free (view->progress_bar);
   ply_label_free (view->label);
+  ply_label_free (view->message_label);
 
   free (view);
 }
@@ -349,11 +358,7 @@ view_show_prompt (view_t     *view,
 
   if (prompt != NULL)
     {
-      int label_width, label_height;
-
       ply_label_set_text (view->label, prompt);
-      label_width = ply_label_get_width (view->label);
-      label_height = ply_label_get_height (view->label);
 
       x = view->box_area.x + view->lock_area.width / 2;
       y = view->box_area.y + view->box_area.height;
@@ -376,6 +381,7 @@ create_plugin (ply_key_file_t *key_file)
 {
   ply_boot_splash_plugin_t *plugin;
   char *image_dir, *image_path;
+  char *color;
 
   srand ((int) ply_get_timestamp ());
   plugin = calloc (1, sizeof (ply_boot_splash_plugin_t));
@@ -394,6 +400,24 @@ create_plugin (ply_key_file_t *key_file)
   plugin->image_dir = image_dir;
   plugin->views = ply_list_new ();
 
+  color = ply_key_file_get_value (key_file, "throbgress", "BackgroundStartColor");
+
+  if (color != NULL)
+    plugin->background_start_color = strtol (color, NULL, 0);
+  else
+    plugin->background_start_color = PLYMOUTH_BACKGROUND_START_COLOR;
+
+  free (color);
+
+  color = ply_key_file_get_value (key_file, "throbgress", "BackgroundEndColor");
+
+  if (color != NULL)
+    plugin->background_end_color = strtol (color, NULL, 0);
+  else
+    plugin->background_end_color = PLYMOUTH_BACKGROUND_END_COLOR;
+
+  free (color);
+
   return plugin;
 }
 
@@ -438,9 +462,13 @@ draw_background (view_t             *view,
   area.width = width;
   area.height = height;
 
-  ply_pixel_buffer_fill_with_gradient (pixel_buffer, &area,
-                                       PLYMOUTH_BACKGROUND_START_COLOR,
-                                       PLYMOUTH_BACKGROUND_END_COLOR);
+  if (plugin->background_start_color != plugin->background_end_color)
+    ply_pixel_buffer_fill_with_gradient (pixel_buffer, &area,
+                                         plugin->background_start_color,
+                                         plugin->background_end_color);
+  else
+    ply_pixel_buffer_fill_with_hex_color (pixel_buffer, &area,
+                                          plugin->background_start_color);
 }
 
 static void
@@ -466,7 +494,7 @@ draw_logo (view_t                   *view,
   view->logo_area.height = height;
 
   ply_pixel_buffer_fill_with_argb32_data (pixel_buffer,
-                                          &view->logo_area, 0, 0,
+                                          &view->logo_area,
                                           logo_data);
 }
 
@@ -530,13 +558,6 @@ stop_animation (ply_boot_splash_plugin_t *plugin,
     ply_trigger_pull (trigger, NULL);
 }
 
-static void
-on_interrupt (ply_boot_splash_plugin_t *plugin)
-{
-  ply_event_loop_exit (plugin->loop, 1);
-  stop_animation (plugin, NULL);
-}
-
 static void
 detach_from_event_loop (ply_boot_splash_plugin_t *plugin)
 {
@@ -552,13 +573,6 @@ on_draw (view_t                   *view,
          int                       height)
 {
   ply_boot_splash_plugin_t *plugin;
-  ply_rectangle_t area;
-
-  area.x = x;
-  area.y = y;
-  area.width = width;
-  area.height = height;
-
   plugin = view->plugin;
 
   draw_background (view, pixel_buffer, x, y, width, height);
@@ -570,13 +584,13 @@ on_draw (view_t                   *view,
 
       box_data = ply_image_get_data (plugin->box_image);
       ply_pixel_buffer_fill_with_argb32_data (pixel_buffer,
-                                              &view->box_area, 0, 0,
+                                              &view->box_area,
                                               box_data);
       ply_entry_draw_area (view->entry, pixel_buffer, x, y, width, height);
       ply_label_draw_area (view->label, pixel_buffer, x, y, width, height);
       lock_data = ply_image_get_data (plugin->lock_image);
       ply_pixel_buffer_fill_with_argb32_data (pixel_buffer,
-                                              &view->lock_area, 0, 0,
+                                              &view->lock_area,
                                               lock_data);
     }
   else
@@ -587,6 +601,9 @@ on_draw (view_t                   *view,
       ply_progress_bar_draw_area (view->progress_bar,
                                   pixel_buffer, x, y, width, height);
     }
+  ply_label_draw_area (view->message_label,
+                       pixel_buffer,
+                       x, y, width, height);
 }
 
 static void
@@ -666,11 +683,6 @@ show_splash_screen (ply_boot_splash_plugin_t *plugin,
                                  detach_from_event_loop,
                                  plugin);
 
-  ply_event_loop_watch_signal (plugin->loop,
-                               SIGINT,
-                               (ply_event_handler_t) 
-                               on_interrupt, plugin);
-
   ply_trace ("starting boot animation");
   start_animation (plugin);
 
@@ -815,6 +827,28 @@ hide_prompt (ply_boot_splash_plugin_t *plugin)
     }
 }
 
+static void
+show_message (ply_boot_splash_plugin_t *plugin,
+              const char               *message)
+{
+  ply_trace ("Showing message '%s'", message);
+  ply_list_node_t *node;
+  node = ply_list_get_first_node (plugin->views);
+  while (node != NULL)
+    {
+      ply_list_node_t *next_node;
+      view_t *view;
+
+      view = ply_list_node_get_data (node);
+      next_node = ply_list_get_next_node (plugin->views, node);
+      ply_label_set_text (view->message_label, message);
+      ply_pixel_display_draw_area (view->display, 10, 10,
+                                   ply_label_get_width (view->message_label),
+                                   ply_label_get_height(view->message_label));
+      node = next_node;
+    }
+}
+
 static void
 display_normal (ply_boot_splash_plugin_t *plugin)
 {
@@ -858,6 +892,13 @@ display_question (ply_boot_splash_plugin_t *plugin,
   unpause_views (plugin);
 }
 
+static void
+display_message (ply_boot_splash_plugin_t *plugin,
+                 const char               *message)
+{
+  show_message (plugin, message);
+}
+
 ply_boot_splash_plugin_interface_t *
 ply_boot_splash_plugin_get_interface (void)
 {
@@ -876,6 +917,7 @@ ply_boot_splash_plugin_get_interface (void)
       .display_normal = display_normal,
       .display_password = display_password,
       .display_question = display_question,      
+      .display_message = display_message,
     };
 
   return &plugin_interface;
